const merge = require('webpack-merge');
const webpack = require('webpack');
const path = require("path");
const FriendlyErrorsWebpackPlugin = require('friendly-errors-webpack-plugin');
const baseConfig = require('./webpack.base.config');

const projectRoot = process.cwd();

const devConfig = {
  mode: "development",
  module: {
    rules: [
      {
        test: /\.css$/,
        use: [
          "style-loader",
          {
            loader: "css-loader",
            options: {
              modules: true,
              localIdentName: "[name]__[local]--[hash:base64:5]"
            }
          }
        ]
      },
      {
        test: /\.scss$/,
        use: [
          "style-loader",
          {
            loader: "css-loader",
            options: {
              modules: true,
              localIdentName: "[name]__[local]--[hash:base64:5]"
            }
          },
          "sass-loader",
          "postcss-loader"
        ]
      }
    ]
  },
  plugins: [
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NamedModulesPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
    new FriendlyErrorsWebpackPlugin()
  ],
  devServer: {
    contentBase: [path.join(projectRoot, "./dist")],
    hot: true,
    host: "0.0.0.0",
    useLocalIp: true,
    port: 8888,
    clientLogLevel: "warning",
    open: true,
    overlay: {
      warnings: true,
      errors: false
    },
    watchOptions: {
      ignored: /node_modules/,
      aggregateTimeout: 100,
      poll: 1000
    }
  },
  // devServer: {
  //   contentBase: "./dist",
  //   hot: true,
  //   stats: "errors-only"
  // },
  devtool: "cheap-module-eval-source-map"
};

module.exports = merge(baseConfig, devConfig);
